package shopping.cart

import akka.actor.typed.ActorSystem
import akka.cluster.sharding.typed.ShardedDaemonProcessSettings
import akka.cluster.sharding.typed.scaladsl.ShardedDaemonProcess
import akka.persistence.cassandra.query.scaladsl.CassandraReadJournal
import akka.persistence.query.Offset
import akka.projection.cassandra.scaladsl.CassandraProjection
import akka.projection.eventsourced.EventEnvelope
import akka.projection.eventsourced.scaladsl.EventSourcedProvider
import akka.projection.scaladsl.{ AtLeastOnceFlowProjection, SourceProvider }
import akka.projection.{ ProjectionBehavior, ProjectionId }

object PublishEventsProjection {

  def init(system: ActorSystem[_]): Unit = {
    val topic =
      system.settings.config.getString("shopping-cart-service.kafka.topic")

    ShardedDaemonProcess(system).init(
      name = "PublishEventsProjection",
      ShoppingCart.tags.size,
      index => ProjectionBehavior(createProjectionFor(system, topic, index)),
      ShardedDaemonProcessSettings(system),
      Some(ProjectionBehavior.Stop))
  }

  private def createProjectionFor(
      system: ActorSystem[_],
      topic: String,
      index: Int): AtLeastOnceFlowProjection[
    Offset,
    EventEnvelope[ShoppingCart.Event]] = { 
    val tag = ShoppingCart.tags(index)
    val sourceProvider
        : SourceProvider[Offset, EventEnvelope[ShoppingCart.Event]] =
      EventSourcedProvider.eventsByTag[ShoppingCart.Event](
        system = system,
        readJournalPluginId = CassandraReadJournal.Identifier, 
        tag = tag)

    CassandraProjection.atLeastOnceFlow( 
      projectionId = ProjectionId("PublishEventsProjection", tag),
      sourceProvider,
      PublishEventsProjectionFlow.createProducerFlow(topic, system))
  }

}
